﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[CreateAssetMenu(fileName = "GameObjectPool", menuName = "Scriptable Object/GameObjectPool", order = 1)]
public class GameObjectPool : ScriptableObject
{
    private GameObject poolObject = null;
    private List<GameObject> pool = new List<GameObject>();

    public int Count { get { return pool.Count; } }

    public GameObject GetAvailable() // Return an available gameobject
    {
        for (int i = 0; i < pool.Count; i++)
        {
            if (!pool[i].activeInHierarchy)
            {
                return pool[i];
            }
        }

        GameObject newObject = Instantiate(poolObject, new Vector2(200, 0), Quaternion.identity); // Spawn new object and add it to the list
        pool.Add(newObject);

        return newObject;
    }

    public void Add(GameObject gameObject)
    {
        if (poolObject == null) // Remember what object is in the pool
        {
            poolObject = gameObject;
        }

        pool.Add(gameObject); // Add the object to the pool
    }
}
