﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
[System.Serializable]
public class BossPhase
{ 

    public string name = "Phase Name";
    public GameObject shot = null;
    public List<GameObject> shotPool = null;
    public int poolCount = 10;
    public Transform poolParent = null;


    public BossPhase()
    {

    }

    public void InitPhase()
    {
        for (int i = 0; i < poolCount; i++)
        {
            Add(shot, poolParent);
        }
    }

    public void Add(GameObject gameObject, Transform parent)
    {
        //if (shot == null) // Remember what object is in the pool
        //{
        //    shot = gameObject;
        //}

        GameObject newObject = MonoBehaviour.Instantiate(gameObject, new Vector2(200, 0), Quaternion.identity, parent);
        newObject.SetActive(false);
        shotPool.Add(newObject); // Add the object to the pool
    }

    public GameObject GetAvailable() // Return an available gameobject
    {
        for (int i = 0; i < shotPool.Count; i++)
        {
            if (!shotPool[i].activeInHierarchy)
            {
                return shotPool[i];
            }
        }

        GameObject newObject = MonoBehaviour.Instantiate(shot, new Vector2(200, 0), Quaternion.identity, poolParent); // Spawn new object and add it to the list
        shotPool.Add(newObject);

        return newObject;
    }
}

public class BossBehaviour : MonoBehaviour
{
    [SerializeField] private ShootingStrategy shootingStrategy = null;
    [SerializeField] private BossPhase shootingPhase = null;

    public enum Phase
    {
        Warmup,
        Serius,
        Intense,
        MadLad,
        End
    }
    public Phase currentPhase = Phase.Warmup;

    private float timer = 0;

    [SerializeField] private BossPhase warmupPhase = new BossPhase();
    [SerializeField] private BossPhase seriousPhase = new BossPhase();
    [SerializeField] private BossPhase intensePhase = new BossPhase();
    [SerializeField] private BossPhase madLadPhase = new BossPhase();

    [SerializeField] private List<Transform> spawnPoints = new List<Transform>();

    public IntVariable bossHP;
    public IntVariable bossMaxHP;
    public Vector3 bossStartPos = new Vector3(0.0f,0.0f,0.0f);
    public Vector3 bossEndPos = new Vector3(0.0f, 0.0f, 0.0f);
    private float bossTravelTime = 0.0f;
    private float bossTravelTimer = 0;
    public bool bossTraveling = false;
    private int bossTravelCounter = 1;

   
    public Vector3 destination1;
    public Vector3 destination2;
    public Vector3 destination3;
    public Vector3 destination4;
    public float mTimer1;
    public float mTimer2;
    public float mTimer3;
    public float mTimer4;

    void Awake()
    {
        bossHP.Variable = 0;
        InitPhases();

        shootingStrategy = new WarmupStrategy();
        shootingPhase = warmupPhase;
        SetDestination(destination1);
        SetMoveSpeed(mTimer1);
    }
    
    void Update()
    {
        if (!BossHpMilestones.interactable)
        {
            shootingStrategy.ShootingBehaviour(spawnPoints, shootingPhase);
            bossMovment();
        }
        timer += Time.deltaTime;
        CheckPhase();
    }

    void SetMoveSpeed(float i)
    {
        bossTravelTime = i;
    }
    
    void SetDestination(Vector3 i)
    {
        bossStartPos = transform.position;
        bossEndPos = i + new Vector3(-4.0f,3.0f,0.0f);
    }

    void bossMovment()
    {
        if (bossTraveling){
            if (bossTravelTime >= bossTravelTimer)
            {
                transform.position = transform.position + ((bossEndPos - bossStartPos) / bossTravelTime * Time.deltaTime);
                bossTravelTimer += Time.deltaTime;
            } else
            {
                bossTravelTimer = 0;
                bossTraveling = false;
            }
        } else
        {
            bossTraveling = true;
            if(bossTravelCounter == 1)
            {
                SetDestination(destination2);
                SetMoveSpeed(mTimer2);
                bossTravelCounter = 2;
            } else if (bossTravelCounter == 2)
            {
                SetDestination(destination3);
                SetMoveSpeed(mTimer3);
                bossTravelCounter = 3;
            } else if (bossTravelCounter == 3)
            {
                SetDestination(destination4);
                SetMoveSpeed(mTimer4);
                bossTravelCounter = 4;
            } else
            {
                SetDestination(destination1);
                SetMoveSpeed(mTimer1);
                bossTravelCounter = 1;
            }
        }
        
    }

    void InitPhases()
    {
        warmupPhase.InitPhase();
        seriousPhase.InitPhase();
        intensePhase.InitPhase();
        madLadPhase.InitPhase();
    }

    void CheckPhase()
    {
        switch(currentPhase)
        {

            case Phase.Warmup:
                if ((bossMaxHP.Variable * 2 / 4) > bossHP.Variable && bossHP.Variable >= (bossMaxHP.Variable / 4))
        {
                    currentPhase = Phase.Serius;
                    shootingStrategy = new FillerStrategy2();
                    shootingPhase = seriousPhase;
                    Debug.Log("<color=red>Switching phase to </color>" + currentPhase);
                }
                break;
            case Phase.Serius:
                if ((bossMaxHP.Variable * 3 / 4) > bossHP.Variable && bossHP.Variable >= (bossMaxHP.Variable * 2 / 4))
        {
                    currentPhase = Phase.Intense;
                    shootingStrategy = new FillerStrategy3();
                    shootingPhase = intensePhase;
                    Debug.Log("<color=red>Switching phase to </color>" + currentPhase);
                }
                break;
            case Phase.Intense:
                if (bossHP.Variable > (bossMaxHP.Variable * 3 / 4))
        {
                    currentPhase = Phase.MadLad;
                    shootingStrategy = new FlowerStrategy();
                    shootingPhase = madLadPhase;
                    Debug.Log("<color=red>Switching phase to </color>" + currentPhase);
                }
                break;
            default:
                break;
        }
    }
}
