﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameManager : MonoBehaviour
{
    [SerializeField] private MainMenu mainMenu = null;
    [SerializeField] private GameObject playingParent = null;
    [SerializeField] private BossHpMilestones hpMilestones = null;

    [SerializeField] private PlayerBehaviour playerBehaviour = null;
    [SerializeField] private SpriteRenderer playerVisual = null;
    [SerializeField] private IntVariable playerLives = null;
    [SerializeField] private Transform spawnpoint = null;
    [SerializeField] private ParticleSystem playerDeathParticle = null;

    [Header("Audio")]
    [SerializeField] private AudioClip menuMusic = null;
    [SerializeField] private AudioClip bossMusic = null;
    [SerializeField] private AudioSource musicSource = null;

    public AudioSource playerAudio = null;
    public AudioClip playerDeathClip = null;

    private bool respawning = false;

    void Start()
    {
        mainMenu.OnBeginPlaying += StartPlaying;
        hpMilestones.OnEndScreen += PlayMainMusic;
        playerBehaviour.OnPlayerHit += PlayerHit;

        musicSource.clip = menuMusic;
        musicSource.Play();

        playerLives.Variable = 3;

        respawning = false;

        playingParent.SetActive(false);
    }

    void OnDestroy()
    {
        mainMenu.OnBeginPlaying -= StartPlaying;
        hpMilestones.OnEndScreen -= PlayMainMusic;
        playerBehaviour.OnPlayerHit -= PlayerHit;
    }

    void StartPlaying()
    {
        musicSource.Stop();
        musicSource.clip = bossMusic;
        musicSource.Play();

        playingParent.SetActive(true);
    }

    void PlayMainMusic()
    {
        musicSource.Stop();
        musicSource.clip = menuMusic;
        musicSource.Play();
    }

    void PlayerHit()
    {
        if (respawning)
            return;

        respawning = true;
        StartCoroutine(TriggerPlayerRespawn());
    }

    IEnumerator TriggerPlayerRespawn()
    {
        //float timer = 0;
        //float respawnTime = 1;
        int blinks = 5;

        //playerBehaviour.gameObject.SetActive(false);
        playerLives.Variable -= 1;

        if (playerLives.Variable > 0)
        {
            //while (timer < respawnTime)
            //{
            //    timer += Time.deltaTime;

            //    yield return null;
            //}
            //playerBehaviour.gameObject.SetActive(true);

            //timer = 0;

            int blinked = 0;
            while (blinked < blinks)
            {
                blinked++;
                playerVisual.enabled = false;
                yield return new WaitForSeconds(0.2f);
                playerVisual.enabled = true;
                yield return new WaitForSeconds(0.2f);
            }
            respawning = false;
        }
        else if (playerLives.Variable == 0)
        {
            playerAudio.PlayOneShot(playerDeathClip);

            playerBehaviour.gameObject.SetActive(false);
            playerDeathParticle.transform.position = playerBehaviour.transform.position;
            playerDeathParticle.Play();
        }
    }
}
