﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class MainMenu : MonoBehaviour
{
    public enum MenuState
    {
        MainMenu,
        Intro,
        Playing,
        EndScreen
    }
    [HideInInspector] public MenuState menuState = MenuState.MainMenu;

    [SerializeField] private GameObject menuParent = null;
    [SerializeField] private GameObject introParent = null;
    [SerializeField] private GameObject playScreenParent = null;
    [SerializeField] private RectTransform background = null;
    [SerializeField] private RectTransform startPosition = null;
    [SerializeField] private RectTransform quitPosition = null;
    [SerializeField] private RectTransform nextPosition = null;
    [SerializeField] private RectTransform arrowPosition = null;

    private CanvasGroup alpha = null;

    [SerializeField] private ParticleSystem selectParticle = null;

    private RectTransform selectedMenuItem = null;

    private int menuIndex = 0;

    private bool interactable = true;

    [Header("Audio")]
    [SerializeField] private AudioSource navigateAudio = null;
    [SerializeField] private AudioSource confirmAudio = null;

    public System.Action OnBeginPlaying;

    private void Start()
    {
        menuParent.SetActive(true);
        introParent.SetActive(false);
        playScreenParent.SetActive(false);

        background.localScale = new Vector3(1, 1, 1);
    }

    void Update()
    {
        if (!interactable)
            return;

        MenuBehaviour();
    }

    void MenuBehaviour()
    {
        switch(menuState)
        {
            case MenuState.MainMenu:
                State_MainMenu();
                break;
            case MenuState.Intro:
                State_Intro();
                break;
            case MenuState.Playing:
                State_Playing();
                break;
            case MenuState.EndScreen:
                State_EndScreen();
                break;
        }
    }

    void State_MainMenu()
    {
        if (Input.GetKeyDown(KeyCode.DownArrow))
        {
            MoveDown();
        }
        else if (Input.GetKeyDown(KeyCode.UpArrow))
        {
            MoveUp();
        }

        if (Input.GetKeyDown(KeyCode.Z))
        {
            interactable = false;

            if (menuIndex == 0)
            {
                StartCoroutine(MenuBlink());
            }
            else
            {
                StartCoroutine(ExitGame());
            }
        }

        UpdateMainMenuVisual();
    }

    void State_Intro()
    {
        if (Input.GetKeyDown(KeyCode.Z))
        {
            interactable = false;
            selectedMenuItem = nextPosition;
            StartCoroutine(MenuBlink());
        }
    }

    void State_Playing()
    {

    }

    void State_EndScreen()
    {

    }

    void UpdateMainMenuVisual()
    {
        if (menuIndex == 0)
        {
            selectedMenuItem = startPosition;
        }
        else
        {
            selectedMenuItem = quitPosition;
        }

        switch(menuIndex)
        {
            case 0:
                arrowPosition.position = startPosition.position;
                break;
            case 1:
                arrowPosition.position = quitPosition.position;
                break;
            default:
                break;
        }
    }

    void MoveDown()
    {
        navigateAudio.Play();

        menuIndex--;
        if (menuIndex < 0)
        {
            menuIndex = 1;
        }
    }

    void MoveUp()
    {
        navigateAudio.Play();

        menuIndex++;
        if (menuIndex > 1)
        {
            menuIndex = 0;
        }
    }

    IEnumerator MenuBlink()
    {
        alpha = selectedMenuItem.GetComponent<CanvasGroup>();

        selectParticle.transform.position = new Vector3(selectedMenuItem.position.x, selectedMenuItem.position.y, selectParticle.transform.position.z);
        selectParticle.Play();
        confirmAudio.Play();

        for (int i = 0; i < 10; i++)
        {
            alpha.alpha = 0f;
            yield return new WaitForSeconds(0.1f);
            alpha.alpha = 1;
            yield return new WaitForSeconds(0.1f);
        }

        selectParticle.Stop();

        if (menuState == MenuState.MainMenu)
        {
            menuParent.SetActive(false);
            introParent.SetActive(true);
            menuState = MenuState.Intro;
        }
        else if (menuState == MenuState.Intro)
        {
            background.localScale = new Vector3(-1, 1, 1);
            introParent.SetActive(false);
            playScreenParent.SetActive(true);
            menuState = MenuState.Playing;

            OnBeginPlaying?.Invoke();
        }

        interactable = true;
    }

    IEnumerator ExitGame()
    {
        alpha = selectedMenuItem.GetComponent<CanvasGroup>();

        selectParticle.Play();
        confirmAudio.Play();

        for (int i = 0; i < 10; i++)
        {
            alpha.alpha = 0f;
            yield return new WaitForSeconds(0.1f);
            alpha.alpha = 1;
            yield return new WaitForSeconds(0.1f);
        }

        selectParticle.Stop();

        Application.Quit();
    }
}
